{% extends "account/base.html" %}

{% load i18n %}
{% load account %}

{% block head_title %}{% trans "Password Reset" %}{% endblock %}

{% block content %}

    <h1>{% trans "Password Reset" %}</h1>
    {% if user.is_authenticated %}
    {% include "account/snippets/already_logged_in.html" %}
    {% endif %}

    <p>{% trans "Forgotten your password? Enter your e-mail address below, and we'll send you an e-mail allowing you to reset it." %}</p>

    <form method="POST" action="{% url 'account_reset_password' %}" class="password_reset">
        {% csrf_token %}
        {{ form.as_p }}
        <input type="submit" value="{% trans 'Reset My Password' %}" />
    </form>

    <p>{% blocktrans %}Please contact us if you have any trouble resetting your password.{% endblocktrans %}</p>
{% endblock %}
